@@ -42,17 +42,26 @@ class FieldPosition (object):
 
     e.g., NE has the ball on the NE 45, the offset is -5.
     e.g., NE has the ball on the NYG 2, the offset is 48.
+
+    This representation allows for gains in any particular play to be added
+    to the field offset to get the new field position as the result of the
+    play.
     """
-    def __new__(cls, pos_team, yardline):
-        if not yardline:
+    def __new__(cls, pos_team=None, yardline=None, offset=None):
+        if not yardline and offset is None:
             return None
         return object.__new__(cls)
 
-    def __init__(self, pos_team, yardline):
+    def __init__(self, pos_team=None, yardline=None, offset=None):
         """
         pos_team is the team on offense, and yardline is a string formatted
         like 'team-territory yard-line'. e.g., "NE 32".
+
+        An offset can be given directly by specifying an integer for offset.
         """
+        if isinstance(offset, int):
+            self.offset = offset
+            return
         if yardline == '50':
             self.offset = 0
             return
@@ -65,10 +74,25 @@ class FieldPosition (object):
             self.offset = 50 - yd
 
     def __cmp__(self, other):
+        if isinstance(other, int):
+            return cmp(self.offset, other)
         return cmp(self.offset, other.offset)
 
     def __str__(self):
-        return '%d' % self.offset
+        if self.offset > 0:
+            return 'OPP %d' % (50 - self.offset)
+        elif self.offset < 0:
+            return 'OWN %d' % (50 + self.offset)
+        else:
+            return 'MIDFIELD'
+
+    def add_yards(self, yards):
+        """
+        Returns a new field position with the yards added to self.
+        Yards may be negative.
+        """
+        newoffset = max(-50, min(50, self.offset + yards))
+        return FieldPosition(offset=newoffset)
 
 
 class PossessionTime (object):
@@ -144,6 +168,20 @@ class GameClock (object):
             else:
                 assert False, 'Unknown QTR value: "%s"' % self.qtr
 
+    @property
+    def quarter(self):
+        return self.__qtr
+
+    @quarter.setter
+    def quarter(self, value):
+        if isinstance(value, int):
+            assert value >= 0 and value <= 4
+            self.qtr = str(value)
+            self.__qtr = value
+        else:
+            self.qtr = value
+            self.__qtr = 0
+
     def is_pregame(self):
         return self.qtr == 'Pregame'
 
@@ -308,8 +346,8 @@ class Game (object):
         Returns a string of the score of the game.
         e.g., "NE (32) vs. NYG (0)".
         """
-        return '%s (%d) vs. %s (%d)' \
-               % (self.home, self.score_home, self.away, self.score_away)
+        return '%s (%d) at %s (%d)' \
+               % (self.away, self.score_away, self.home, self.score_home)
 
     def max_player_stats(self):
         """
@@ -461,11 +499,21 @@ class Drive (object):
                 self.field_end = FieldPosition(self.team, '50')
 
         # When a drive lasts from Q1 to Q2 or Q3 to Q4, the 'end' doesn't
-        # seem to change to the proper quarter. So look at the last play and
-        # use that quarter instead.
-        lastplayid = str(sorted(map(int, data['plays'].keys()))[-1])
-        endqtr = data['plays'][lastplayid]['qtr']
-        self.time_end = GameClock(endqtr, data['end']['time'])
+        # seem to change to the proper quarter. So scan all of the plays
+        # and use the maximal quarter listed. (Just taking the last doesn't
+        # seem to always work.)
+        # lastplayid = str(max(map(int, data['plays'].keys())))
+        # endqtr = data['plays'][lastplayid]['qtr']
+        maxq = str(max(map(int, [p['qtr'] for p in data['plays'].values()])))
+        self.time_end = GameClock(maxq, data['end']['time'])
+
+        # One last sanity check. If the end time is less than the start time,
+        # then bump the quarter if it seems reasonable.
+        # This technique will blow up if a drive lasts more than fifteen
+        # minutes and the quarter numbering is messed up.
+        if self.time_end <= self.time_start \
+                and self.time_end.quarter in (1, 3):
+            self.time_end.quarter += 1
 
         self.__plays = _json_plays(self, data['plays'])
         self.plays = nflgame.seq.GenPlays(self.__plays)
@@ -618,16 +666,7 @@ def _json_drives(game, home_team, data):
         except:
             pass
     drives = []
-    playids = set()  # Plays can be repeated! Ah!
     for i, drive_num in enumerate(sorted(drive_nums), 1):
-        repeat_drive = False
-        for playid in data[str(drive_num)]['plays']:
-            if playid in playids:
-                repeat_drive = True
-                break
-            playids.add(playid)
-        if repeat_drive:
-            continue
         drives.append(Drive(game, i, home_team, data[str(drive_num)]))
     return drives
 
@@ -635,10 +674,19 @@ def _json_drives(game, home_team, data):
 def _json_plays(drive, data):
     """
     Takes a single JSON drive entry (data) and converts it to a list
-    of Play objects.
+    of Play objects. This includes trying to resolve duplicate play
+    conflicts by only taking the first instance of a play.
     """
     plays = []
+    seen_ids = set()
+    seen_desc = set()  # Sometimes duplicates have different play ids...
     for playid in map(str, sorted(map(int, data))):
+        p = data[playid]
+        desc = (p['desc'], p['time'], p['yrdln'], p['qtr'])
+        if playid in seen_ids or desc in seen_desc:
+            continue
+        seen_ids.add(playid)
+        seen_desc.add(desc)
         plays.append(Play(drive, playid, data[playid]))
     return plays
 
